import aiohttp
from typing import Optional

class ReposAPI:
    def __init__(self, client: aiohttp.ClientSession, token: str, base_url: str):
        self._client = client
        self.token = token
        self.base_url = base_url

    async def add_repo(self, url: str):
        """Suggest repo(add repo if admin)"""
        payload = {
            "url": url,
            "token": self.token
        }
        async with self._client.post(f"{self.base_url}/repos/add", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data

    async def remove_repo(self, url: str, reason: None | str = None):
        """Remove repo(ADMIN ONLY)"""
        payload = {
            "url": url,
            "token": self.token,
        }
        if reason:
            payload["reason"] = reason
        
        async with self._client.delete(f"{self.base_url}/repos/remove", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data
    
    async def scan(self):
        """Rescan repos for new modules(ADMIN ONLY)"""
        payload = {
            "token": self.token
        }

        data = []
        async with self._client.get(f"{self.base_url}/repos/scan", params=payload) as response:
            response.raise_for_status()
            async for line in response.content:
                if line:
                    line_str = line.decode('utf-8').strip()
                    if line_str.startswith("data: "):
                        data.append(line_str[6:])
            
            return data


import aiohttp
from typing import Optional

class ModulesAPI:
    def __init__(self, client: aiohttp.ClientSession, token: str, base_url: str):
        self._client = client
        self.token = token
        self.base_url = base_url
    
    async def get_rate(self, module_id: str):
        """Get rate for module with given id"""
        payload = {
            "module_id": module_id
        }
        async with self._client.get(f"{self.base_url}/modules/rate", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data
    
    async def search(self, query: str):
        """Search modules by query"""
        payload = {
            "query": query
        }
        async with self._client.get(f"{self.base_url}/modules/search", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data
    
    async def modules(self):
        """Get all modules(ADMIN ONLY)"""
        payload = {
            "token": self.token
        }
        async with self._client.get(f"{self.base_url}/modules/modules", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data
    
    async def module(self, module_id: int):
        """Get info about module by module_id"""
        async with self._client.get(f"{self.base_url}/modules/{module_id}") as response:
            response.raise_for_status()
            data = await response.json()
            return data

    async def rnd(self, count: int):
        """Get random module"""
        payload = {
            "count": count
        }
        async with self._client.get(f"{self.base_url}/modules/rnd", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data
    
    async def rate_module(self, module_id: int, rating: str):
        """Rate module(like or dislike)"""
        val = rating.lower()
        if val not in ["like", "dislike"]:
            raise ValueError(detail="Only like or dislike allowed")
        
        payload = {
            "module_id": module_id,
            "rating": val,
            "token": self.token
        }
        
        async with self._client.post(f"{self.base_url}/modules/rate", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data


import aiohttp
from typing import Optional

class CheckerAPI:
    def __init__(self, client: aiohttp.ClientSession, token: str, base_url: str):
        self._client = client
        self.token = token
        self.base_url = base_url

    async def verdict(self, source: str, hash: str, status: str, reason: Optional[str] = None):
        """Set verfict for something(ADMIN ONLY)"""
        payload = {
            "token": self.token,
            "source": source,
            "hash": hash,
            "status": status,
            "reason": reason,
        }
        async with self._client.post(f"{self.base_url}/checker/verdict", params=payload) as response:
            response.raise_for_status()
            data = await response.json()
            return data